package Bio::Phylo::Util::CONSTANT::Int;

use strict;
use warnings;

sub _NONE_ ()          { 1 }
sub _NODE_ ()          { 2 }
sub _TREE_ ()          { 3 }
sub _FOREST_ ()        { 4 }
sub _TAXON_ ()         { 5 }
sub _TAXA_ ()          { 6 }
sub _DATUM_ ()         { 7 }
sub _MATRIX_ ()        { 8 }
sub _MATRICES_ ()      { 9 }
sub _SEQUENCE_ ()      { 10 }
sub _ALIGNMENT_ ()     { 11 }
sub _CHAR_ ()          { 12 }
sub _PROJECT_ ()       { 9 }
sub _CHARSTATE_ ()     { 13 }
sub _CHARSTATESEQ_ ()  { 14 }
sub _MATRIXROW_ ()     { 15 }
sub _ANNOTATION_ ()    { 16 }
sub _DICTIONARY_ ()    { 17 }
sub _DOMCREATOR_ ()    { 18 }
sub _META_ ()          { 19 }
sub _DESCRIPTION_ ()   { 20 }
sub _RESOURCE_ ()      { 21 }
sub _DOCUMENT_ ()      { 22 }
sub _ELEMENT_ ()       { 23 }
sub _CHARACTERS_ ()    { 24 }
sub _CHARACTER_ ()     { 25 }
sub _SET_ ()           { 26 }
sub _MODEL_ ()         { 27 }
sub _OPERATION_ ()     { 28 }
sub _DATATYPE_ ()      { 29 }

=head1 NAME

Bio::Phylo::Util::CONSTANT::Int - Integer constants, no serviceable parts inside

=cut

1;
